import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:pmu/data/repositories/bbc_repository.dart';
import 'package:pmu/presentation/home_page/bloc/events.dart';
import 'package:pmu/presentation/home_page/bloc/state.dart';
//класс для управления состоянием главной страницы приложения, взаимодействует с репозиторием, отвечает за подгрузку данных
class HomeBloc extends Bloc<HomeEvent, HomeState> {
  // объект для доступа к репозиторию
  final BbcRepository repo;

  HomeBloc(this.repo) : super(const HomeState()) {
    on<HomeLoadDataEvent>(_onLoadData);
  }

  // мы должны изменить наше состояние, передаем внутрь новое состояние
  Future<void> _onLoadData(HomeLoadDataEvent event, Emitter<HomeState> emit) async {
    // изменение состояния загрузки
    if (event.nextPage == null) {
      emit(state.copyWith(isLoading: true));
    } else {
      emit(state.copyWith(isPaginationLoading: true));
    }
    //загрузка данных
    String? error;
    final data = await repo.loadData(
      q: event.search,
      page: event.nextPage ?? 5,
      onError: (e) => error = e,
    );
    //объединение новых данных со старыми
    if (event.nextPage != null) {
      data?.data?.insertAll(0, state.data?.data ?? []);
    }
    // эмит нового состояния
    emit(state.copyWith(
      isLoading: false,
      isPaginationLoading: false,
      data: data,
      error: error,
    ));
  }
}
